{{
        Test the Continuous Timestamping reciprocal frequency counter

        Jonathan "lonesock" Dummer
}}
CON
  _CLKMODE = XTAL1 + PLL16X
  _XINFREQ = 5_000_000

  ' serial pins
  term_RX       = 31
  term_TX       = 30
  term_bps      = 460_800

  test_pin      = 4


OBJ
  term:         "FFDS1"
  fc:           "CTS_FreqCounter"

dat
  spinidx byte 0
  spinchars byte "|/-\"

PRI frac32( num, den )
  repeat 32
    result += result
    if num => den
      result += 1
      num -= den
    num += num

PUB Main | counts, Pdrive, fdrive, fs, fnum, fden, {
          } inverse_ratio ', ir_debug_val

  ' set up counter A to do a NCO squarewave out
  ctra := constant( %00100 << 26 )     ' PLL output, single ended
  ctra |= test_pin                     ' output pin
  ' set a frequency (CTS seems to handle just up to 2.35 MHz)
  fs := 100
  ' The equation would look something like:
  '   frqa = fs * (1 << 32) / clkfreq
  ' But that is numerically horrible to do in integer math
  frqa := frac32( fs * 2, clkfreq )
  dira[ test_pin ] := true

  ' start up the frequency counter
  fc.start( test_pin, clkfreq / 100, @inverse_ratio )
    
  ' Start the serial link
  if ina[term_RX]
    ' the RX pin is high, so there is probably power to the FTDI chip
    term.start( term_RX, term_TX, term_bps )
  else
    ' just RX, since the FTDI is probably unpowered
    term.start( term_RX, -1, term_bps )  

  '{ START SWEEP
  waitcnt( clkfreq * 2 + cnt )
  term.tx( 0 )
  term.str( string( 13, 13, "FreqOut FreqMeas Err", 13 ) )

  ' limit at about 2_353_500
  repeat fdrive from 2_352_000 to 2_354_000 step 10
    'repeat fdrive from 0_100_000 to 2_500_000 step 10_000
    ' set the frequency
    frqa := frac32( fdrive * 2, clkfreq )
    ' wait a bit
    waitcnt( clkfreq / 10 + cnt )

    ' new version
    fs := clkfreq ** inverse_ratio

    ' send back results
    term.dec( fdrive )
    term.tx( " " )
    term.dec( fs )
    term.tx( " " )
    term.dec( fdrive - fs )
    term.tx( 13 )

  'waitpne( 0,0,0 )
  term.rx
  ' END SWEEP }


  '{ set up counter A to do a PLL squarewave out, trying for NOT synced to the system clock
  ctra := constant( %00010 << 26 )     ' PLL output, single ended
  ctra |= constant( %000 << 23 )       ' VCO / 128 (and VCO is 16x the NCO frequency, so NCO / 8)
  ctra |= test_pin                     ' output pin
  ' set a frequency (docs say 4MHz to 8MHz to be stable, limit seems right around 14MHz)
  fs := 6_000_000 ' 4_000_000 to 8_000_000
  ' The equation would look something like:
  '   frqa = fs * (1 << 32) / clkfreq
  ' But that is numerically horrible to do in integer math
  frqa := frac32( fs * 2, clkfreq )
  ' OR...just go all the way
  frqa := posx ' WAY outside the stable range for the PLL!!   
  dira[ test_pin ] := true
  '}


  ' monitor period
  repeat
    waitcnt( clkfreq / 4 + cnt )

    ' result back as an integer
    fs := clkfreq ** inverse_ratio
    term.tx( 0 ) ' good for PropSerTerm
    'term.str( string( 8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8 ) ) ' good for BST term
    term.tx( spinchars[ ++spinidx & 3 ] )
    term.tx( " " )
    term.dec( fs )
    'term.tx( " " )
    'term.dec( ir_debug_val )